package com;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;

import com.common.Analyzer;
import com.common.FileBox;
import com.common.MailSeans;

/*
 * Created on 12.12.2004
 */

/**
 * @author nemo
 */
public class ExecThread extends Thread {

	private static final int MIN_LENGTH = 2;

	private MailChecker _mainPan;

	private Object[] _files;

	private ArrayList<MailSeans> _rawSeanses;

	private boolean isStop = false;

	HashMap<String, MailSeans> seansMap = new HashMap<String, MailSeans>();

	HashMap<String, MailSeans> deliveryQueueMap = new HashMap<String, MailSeans>();

	public void setStop() {
		isStop = true;
	}

	private String getFileContent(File fitem) {
		try {
			FileInputStream fin = new FileInputStream(fitem);
			byte[] content = { 0 };
			try {
				content = new byte[fin.available()];
				fin.read(content);
				fin.close();
			} catch (IOException e1) {
				e1.printStackTrace();
				return "";
			}
			return new String(content);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		return "";
	}

	public ExecThread(MailChecker mainPan) {
		_mainPan = mainPan;
		_rawSeanses = _mainPan.getDataSave().getRawData();
		_files = _mainPan.getDataSave().getFileRows().toArray();
		if (_files == null) {
			_mainPan.clean();
			return;
		}
		start();
	}

	public void run() {
		//String _data;
		FileBox fbox;
		int numFiles = _files.length;
		for (int i = 0; i < numFiles; i++) {
			if (isStop)
				return;
			fbox = (FileBox) _files[i];
			if (!fbox.isAnalyze())
				continue;
			_mainPan.getProgressBar().setStatustext("Loading ... " + fbox.getFile().getName());
			processDataStrings(fbox.getFile());
			fbox.setAnalyzed(false);
			System.gc();
		}
		_mainPan.getProgressBar().setStatustext("Allmost Done....");
		_mainPan.getDataSave().flushUsers();
		_mainPan._value = 0;
		_mainPan.getProgressBar().setStatustext("Done.");
		_mainPan.showResultUsers();
		System.gc();
	}

	private void processDataStrings(File fitem) {
		_mainPan._value = 0;
		_mainPan._max = fitem.length();
		try {
			BufferedReader in = new BufferedReader(new FileReader(fitem));
			String strLine = new String();
			String startToken = " qmail: ";
			int posQmail = 0;
			while ((strLine = in.readLine()) != null) {
				if (isStop)
					return;
				posQmail = strLine.indexOf(startToken);
				if (posQmail != -1)
					parseLogLine(strLine.substring(posQmail + startToken.length()));
				else
					parseLogLine(strLine);
				_mainPan._value += strLine.length();
			}
			in.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
		System.gc();
	}

	private void parseLogLine(String logStr) {
		logStr = logStr.toLowerCase();
		if (isNewMsg(logStr))
			return;
		if (isInfoMsg(logStr))
			return;
		if (isStartingDelivery(logStr))
			return;
		if (isSuccessDelivery(logStr))
			return;
		isEndMsg(logStr);
	}

	private void isEndMsg(String logStr) {
		try {
			String[] tokens = logStr.split(" ");
			if ((tokens[1].equals("end") && (tokens[2].equals("msg")) && (seansMap.containsKey(tokens[3] + ":")))) {
				seansMap.remove(tokens[3] + ":");
			}
		} catch (Exception e) {
			//System.out.println("isNewMsg---"+logStr);
			//e.printStackTrace();
		}
	}

	private boolean isSuccessDelivery(String logStr) {
		try {
			String[] tokens = logStr.split(" ");
			if ((tokens[1].equals("delivery") && (tokens[3].equals("success:")) && (deliveryQueueMap
					.containsKey(tokens[2])))) {
				MailSeans seans = deliveryQueueMap.get(tokens[2]);
				deliveryQueueMap.remove(tokens[2]);
				if ((seans.from.length() > MIN_LENGTH) && (seans.fromDomain.length() > MIN_LENGTH)
						&& (seans.to.length() > MIN_LENGTH) && (seans.toDomain.length() > MIN_LENGTH)) {
					_rawSeanses.add(seans);
					Analyzer.putSeansRow(_mainPan.getDataSave(), seans);
				}
				return true;
			}
		} catch (Exception e) {
			//System.out.println("isSuccessDelivery---"+logStr);
			//e.printStackTrace();
		}
		return false;
	}

	private boolean isStartingDelivery(String logStr) {
		try {
			String[] tokens = logStr.split(" ");
			if ((tokens[1].equals("starting") && (tokens[2].equals("delivery")) && (seansMap.containsKey(tokens[5]
					+ ":")))) {
				MailSeans seans = seansMap.get(tokens[5] + ":");
				seans = seans.clone();
				seans.deliveryQueueId = tokens[3];
				deliveryQueueMap.put(seans.deliveryQueueId, seans);
				String[] address = tokens[8].split("@");
				seans.to = address[0];
				seans.toDomain = address[1].replace("\r", "");
				seans.to = seans.to.replace(seans.toDomain + "-", "");
				return true;
			}
		} catch (Exception e) {
			//System.out.println("isStartingDelivery---"+logStr);
			//e.printStackTrace();
		}
		return false;
	}

	private boolean isInfoMsg(String logStr) {
		try {
			String[] tokens = logStr.split(" ");
			if ((tokens[1].equals("info") && (tokens[2].equals("msg")) && (seansMap.containsKey(tokens[3])))) {
				MailSeans seans = seansMap.get(tokens[3]);
				seans.bytes = Integer.valueOf(tokens[5]).intValue();
				String[] address = tokens[7].split("@");
				if (address.length < 2)
					return false;
				seans.from = address[0];
				seans.fromDomain = address[1];
				seans.from = seans.from.replace(address[1] + "-", "");
				seans.from = seans.from.replace("<", "");
				seans.fromDomain = seans.fromDomain.replace(">", "");
				return true;
			}
		} catch (Exception e) {
			//System.out.println("isInfoMsg---"+logStr);
			//e.printStackTrace();
		}
		return false;
	}

	private boolean isNewMsg(String logStr) {
		try {
			String[] tokens = logStr.split(" ");
			if ((tokens[1].equals("new") && (tokens[2].equals("msg")))) {
				MailSeans seans = new MailSeans();
				seans.messId = tokens[3].replace("\r", "") + ":";
				seansMap.put(seans.messId, seans);
				return true;
			}
		} catch (Exception e) {
			//System.out.println("isNewMsg---"+logStr);
			//e.printStackTrace();
		}
		return false;
	}

}
