/*
 * Created on 24.12.2004
 */
package com.common;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Polygon;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.swing.Icon;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumnModel;


/**
 * @author nemo
 */
public class AdvancedTableModel extends AbstractTableModel {
	/**
	 * Comment for <code>serialVersionUID</code>
	 */
	private static final long serialVersionUID = 3258128055272026169L;
	public static final int DESCENDING = -1;
	public static final int NOT_SORTED = 0;
	public static final int ASCENDING = 1;
	private static Directive EMPTY_DIRECTIVE = new Directive(-1, NOT_SORTED);
	
	public static final Comparator COMPARABLE_COMAPRATOR = new Comparator() {
		public  int compare (Object o1, Object o2) {
			return ((Comparable) o1).compareTo(o2);
		}
	};

	public static final Comparator LEXICAL_COMPARATOR = new Comparator() {
		public int compare(Object o1, Object o2) {
			//System.out.println(o1.toString()+"------lex------"+o2.toString());
			return o1.toString().compareTo(o2.toString());
		}
	};

	private Row[] viewToModel;

	private int[] modelToView;

	private JTableHeader tableHeader;

	private MouseListener mouseListener;

	private List <Directive> sortingColumns = new ArrayList <Directive> ();

    private Map<Class , Comparator>  columnComparators = new HashMap <Class , Comparator> ();
    
	private ArrayList <TableRow> _filteredRows;
	
	private  JTable _table;
	
	private String[] _columnNames;

	public AdvancedTableModel( ArrayList  <TableRow> filteredRows, String[] columnNames) {
		_filteredRows = filteredRows;
		_columnNames = columnNames;
		this.mouseListener = new MouseHandler();
		//this.tableModelListener = new TableModelHandler();
		 new TableModelHandler();
	}
	
	public void setTable(JTable table) {
		_table = table;
		setTableHeader(table.getTableHeader());
	}

	private void clearSortingState() {
		viewToModel = null;
		modelToView = null;
	}

	public JTableHeader getTableHeader() {
		return tableHeader;
	}

	public void setTableHeader(JTableHeader tableHeader) {
		if (this.tableHeader != null) {
			this.tableHeader.removeMouseListener(mouseListener);
			TableCellRenderer defaultRenderer = this.tableHeader.getDefaultRenderer();
			if (defaultRenderer instanceof SortableHeaderRenderer) {
				this.tableHeader.setDefaultRenderer(((SortableHeaderRenderer) defaultRenderer).tableCellRenderer);
			}
		}
		this.tableHeader = tableHeader;
		if (this.tableHeader != null) {
			this.tableHeader.addMouseListener(mouseListener);
			this.tableHeader.setDefaultRenderer(new SortableHeaderRenderer(this.tableHeader.getDefaultRenderer()));
		}
	}

	public boolean isSorting() {
		return sortingColumns.size() != 0;
	}

	private Directive getDirective(int column) {
		for (int i = 0; i < sortingColumns.size(); i++) {
			Directive directive = sortingColumns.get(i);
			if (directive.column == column) {
				return directive;
			}
		}
		return EMPTY_DIRECTIVE;
	}

	public int getSortingStatus(int column) {
		return getDirective(column).direction;
	}

	private void sortingStatusChanged() {
		clearSortingState();
		fireTableDataChanged();
		tableHeader.repaint();
	}

	public void setSortingStatus(int column, int status) {
		Directive directive = getDirective(column);
		if (directive != EMPTY_DIRECTIVE) {
			sortingColumns.remove(directive);
		}
		if (status != NOT_SORTED) {
			sortingColumns.add(new Directive(column, status));
		}
		sortingStatusChanged();
	}

	protected Icon getHeaderRendererIcon(int column, int size) {
		Directive directive = getDirective(column);
		return new Arrow(directive.direction, size, sortingColumns.indexOf(directive));
	}

	private void cancelSorting() {
		sortingColumns.clear();
		sortingStatusChanged();
	}

	public void setColumnComparator(Class type, Comparator comparator) {
		if (comparator == null) {
			columnComparators.remove(type);
		} else {
			columnComparators.put(type, comparator);
		}
	}

	protected Comparator  getComparator(int column) {
		Class columnType = getColumnClass(column);
		Comparator comparator = columnComparators.get(columnType);
		if (comparator != null) {
			return comparator;
		}
		if (Comparable.class.isAssignableFrom(columnType)) {
			return COMPARABLE_COMAPRATOR;
		}
		return LEXICAL_COMPARATOR;
	}

	private Row[] getViewToModel() {
		if (viewToModel == null) {
			int rowCount = getRowCount();
			viewToModel = new Row[rowCount];
			for (int row = 0; row < rowCount; row++) {
				viewToModel[row] = new Row(row,_filteredRows.get(row));
			}
			if (isSorting()) {
				Arrays.sort(viewToModel);
			}
		}
		return viewToModel;
	}

	public int modelIndex(int viewIndex) {
		return getViewToModel()[viewIndex].modelIndex;
	}

	private int[] getModelToView() {
		if (modelToView == null) {
			int n = getViewToModel().length;
			modelToView = new int[n];
			for (int i = 0; i < n; i++) {
				modelToView[modelIndex(i)] = i;
			}
		}
		return modelToView;
	}
	
	// TableModel interface methods 

	public int getRowCount() {
		return _filteredRows.size();
	}

	public int getColumnCount() {
		return _columnNames.length;
	}

	public String getColumnName(int column) {
		return _columnNames[column];
	}

	public Class <?> getColumnClass(int column) {
		TableRow drow= null;
		if (column == 0)
			return Integer.class;
		try {
			drow =  _filteredRows.get(0);
		} catch (RuntimeException e) {
			e.printStackTrace();
		}
		if(drow == null)
			return String.class;
		return drow.getValueAt(column).getClass();
	}

	public boolean isCellEditable(int row, int column) {
		return false;
	}

	private void setRowColors(int row, int column, TableRow drow) {
		try {
			JLabel defRenderer;
			if (column == 0)
				defRenderer = ((JLabel) _table.getCellRenderer(row, column)
						.getTableCellRendererComponent(_table,new Integer(row + 1), false, false, row, column));
			else
			defRenderer = ((JLabel) _table.getCellRenderer(row, column)
							.getTableCellRendererComponent(_table,drow.getValueAt(column), false, false, row, column));
			if(drow.getRowColor()!= null) {
				defRenderer.setBackground(drow.getRowColor());
				return;
			}
			if ((row % 2) > 0) {
				defRenderer.setBackground(new Color(210, 245, 210));
			} else {
				defRenderer.setBackground(Color.WHITE);
			}
		} catch (RuntimeException e) {
			e.printStackTrace();
		}
	}

	public Object getValueAt(int row, int column) {
		if ((viewToModel != null) && (viewToModel.length != getRowCount()))
			viewToModel = null;
		try {
			TableRow drow =  _filteredRows.get(modelIndex(row));
			setRowColors(row, column, drow);
			if (column == 0)
				return new Integer(row + 1);
			if (drow != null)
				return drow.getValueAt(column);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return "+";
	}

	public void setValueAt(Object aValue, int row, int column) {
		if ((viewToModel != null) && (viewToModel.length != getRowCount()))
			viewToModel = null;
		TableRow drow =  _filteredRows.get(modelIndex(row));
		drow.setValueAt(aValue, column);
	}

	// Helper classes
	
	private class Row implements Comparable {
		private int modelIndex;
		private TableRow _tableRow;

		public Row(int index,TableRow tableRow) {
			this.modelIndex = index;
			this._tableRow = tableRow;
		}

		public int compareTo(Object o) {
			TableRow drow1 =  _tableRow;
			TableRow drow2 =  ((Row) o)._tableRow;
			int comparison = drow1.compareRows(drow2);
			if (comparison != 0) {
				return comparison;
			}

			for (Iterator it = sortingColumns.iterator(); it.hasNext();) {
				Directive directive = (Directive) it.next();
				int column = directive.column;
				Object o1 = drow1.getValueAt(column);
				Object o2 = drow2.getValueAt(column);
				// Define null less than everything, except null.
				if (o1 == null && o2 == null) {
					comparison = 0;
				} else if (o1 == null) {
					comparison = -1;
				} else if (o2 == null) {
					comparison = 1;
				} else {
					Comparator comp = getComparator(column);
					comparison = comp.compare(o1, o2);
				}
				if (comparison != 0) {
					return directive.direction == DESCENDING ? -comparison : comparison;
				}
			}
			return 0;
		}
	}

	private static class Arrow implements Icon {

		private int size;

		private int dir;

		public Arrow(int _dir, int size, int priority) {
			dir = _dir;
			this.size = size;
		}

		public void paintIcon(Component c, Graphics g, int x, int y) {
			Color color = new Color(94, 106, 251);
			// In a compound sort, make each succesive triangle 20% 
			// smaller than the previous one. 
			//int dx = (int) (size / 2 * Math.pow(0.8, priority));
			//int dy = descending ? dx : -dx;
			int dx = size - 5;
			int dy = dx;
			// Align icon (roughly) with font baseline. 
			//y = y + 5 * size / 6 + (descending ? -dy : 0);
			y = y + 2;
			x += 3;
			//int shift = descending ? 1 : -1;
			g.translate(x, y);
			g.setColor(color);
			if (dir == NOT_SORTED) {
				g.fillRect(0, 0, dy, dy);
				g.setColor(color.brighter());
				g.drawRect(0, 0, dy, dy);
			} else if (dir == ASCENDING) {
				int[] xpoints = { 0, dx, dx / 2 };
				int[] ypoints = { 0, 0, dy };
				Polygon triangle = new Polygon(xpoints, ypoints, 3);
				g.fillPolygon(triangle);
				g.setColor(color.brighter());
				g.drawPolygon(triangle);
			} else if (dir == DESCENDING) {
				int[] xpoints = { 0, dx / 2 + 1, dx + 1 };
				int[] ypoints = { dy, 0, dy };
				Polygon triangle = new Polygon(xpoints, ypoints, 3);
				g.fillPolygon(triangle);
				g.setColor(color.brighter());
				g.drawPolygon(triangle);
			}
			g.translate(-x, -y);
		}

		public int getIconWidth() {
			return size;
		}

		public int getIconHeight() {
			return size;
		}
	}

	private class TableModelHandler implements TableModelListener {
		public void tableChanged(TableModelEvent e) {
			if (!isSorting()) {
				fireTableChanged(e);
				return;
			}
			if (e.getFirstRow() == TableModelEvent.HEADER_ROW) {
				cancelSorting();
				fireTableChanged(e);
				return;
			}
			int column = e.getColumn();
			if (e.getFirstRow() == e.getLastRow() && column != TableModelEvent.ALL_COLUMNS
					&& getSortingStatus(column) == NOT_SORTED && modelToView != null) {
				int viewIndex = getModelToView()[e.getFirstRow()];
				fireTableChanged(new TableModelEvent(AdvancedTableModel.this, viewIndex, viewIndex, column, e.getType()));
				return;
			}
			clearSortingState();
			fireTableDataChanged();
			return;
		}
	}

	private class MouseHandler extends MouseAdapter {
		public void mouseClicked(MouseEvent e) {
			JTableHeader h = (JTableHeader) e.getSource();
			TableColumnModel columnModel = h.getColumnModel();
			int viewColumn = columnModel.getColumnIndexAtX(e.getX());
			int column = columnModel.getColumn(viewColumn).getModelIndex();
			if (column != -1) {
				int status = getSortingStatus(column);
				if (!e.isControlDown()) {
					cancelSorting();
				}
				status = status + (e.isShiftDown() ? -1 : 1);
				status = (status + 1) % 3 - 1; // signed mod returning {-1, 0, 1}
				setSortingStatus(column, status);
			}
		}
	}

	private class SortableHeaderRenderer implements TableCellRenderer {
		private TableCellRenderer tableCellRenderer;

		public SortableHeaderRenderer(TableCellRenderer tableCellRenderer) {
			this.tableCellRenderer = tableCellRenderer;
		}

		public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected,
				boolean hasFocus, int row, int column) {
			Component c = tableCellRenderer.getTableCellRendererComponent(table, value, isSelected, hasFocus, row,
					column);
			if (c instanceof JLabel) {
				JLabel l = (JLabel) c;
				l.setHorizontalTextPosition(JLabel.LEFT);
				int modelColumn = table.convertColumnIndexToModel(column);
				l.setIcon(getHeaderRendererIcon(modelColumn, l.getFont().getSize()));
			}
			return c;
		}
	}

	private static class Directive {
		private int column;

		private int direction;

		public Directive(int column, int direction) {
			this.column = column;
			this.direction = direction;
		}
	}
}

