package com.common;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.KeyEvent;
import java.awt.print.PrinterException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;

import com.MailChecker;
import com.ToolBarWrapper;
import com.dialogs.TextEditDialog;

/*
 * Created on 13.12.2004
 */

/**
 * @author nemo
 */
public class TablePanel extends JPanel {

	private static final int COLUMN_SPACE = 2;
    /**
	 * Comment for <code>serialVersionUID</code>
	 */
	private static final long serialVersionUID = 3258130254312060977L;

	private String[] _columnNames;

	private JTable table;

	private JTextField filterField = new JTextField("");

	private ArrayList<TableRow> rows = new ArrayList<TableRow>();

	private ArrayList<TableRow> filteredRows = new ArrayList<TableRow>();

	private TotalFactory _totalFactory;
	
	private MailChecker _mainPanel;

	//private TableStatusBar _sbar;

	private UserBox totalMb;

	public void filterRows(int colNum) {
		if (rows.size() == 0)
			return;
		Iterator i = rows.iterator();
		UserBox row;
		filteredRows.clear();
		Pattern patt;
		try {
			patt = Pattern.compile(".*" + filterField.getText().trim() + ".*");
		} catch (RuntimeException e1) {
			e1.printStackTrace();
			patt = Pattern.compile(".*");
		}
		Matcher m;
		UserBox totalBytes = new UserBox("Total Bytes: ", null);
		totalMb = (UserBox) _totalFactory.createTotal();
		totalMb.setFooter(2);
		while (i.hasNext()) {
			row = (UserBox) i.next();
			try {
				m = patt.matcher(row.getValueAt(colNum).toString());
				if (m.matches()) {
					filteredRows.add(row);
					totalBytes.setIncoming(new Float(totalBytes.getIncoming().floatValue()
							+ row.getIncoming().floatValue()));
					totalBytes.setOutgoing(new Float(totalBytes.getOutgoing().floatValue()
							+ row.getOutgoing().floatValue()));
					totalBytes.setLocalIn(new Float(totalBytes.getLocalIn().floatValue()
							+ row.getLocalIn().floatValue()));
					totalBytes.setLocalOut(new Float(totalBytes.getLocalOut().floatValue()
							+ row.getLocalOut().floatValue()));
				}
			} catch (RuntimeException e) {
				filteredRows.add(row);
				//e.printStackTrace();
				//return;
			}
		}
		//filteredRows.add(totalBytes);
		totalMb.setIncoming(new Float((totalBytes.getIncoming().floatValue() / 1024 / 1024)));
		totalMb.setOutgoing(new Float((totalBytes.getOutgoing().floatValue() / 1024 / 1024)));
		totalMb.setLocalIn(new Float((totalBytes.getLocalIn().floatValue() / 1024 / 1024)));
		totalMb.setLocalOut(new Float((totalBytes.getLocalOut().floatValue() / 1024 / 1024)));
		totalMb.setRowColor(new Color(255, 200, 200));
		//_sbar._rows.setText(" "+filteredRows.size());
		//_sbar._totalIn.setText(new Float(totalMb.incoming).toString());
		//_sbar._rows.setText(new Float(totalMb.outgoing).toString());
		//System.out.println("-------------------------"+filteredRows.size()+"--------------"+totalMb.incoming+"----------"+totalMb.outgoing);
		filteredRows.add(totalMb);
	}

	public void printTable() {
		try {
			table.print();
		} catch (PrinterException e) {
			e.printStackTrace();
		}
	}

	public String saveTableToText() {
		try {
			int[] maxLen = getMaxLengthContent();
			StringBuffer buff = new StringBuffer("");
            fillHeader(buff,maxLen);
			fillRows(maxLen, buff);
			return buff.toString();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return "";
	}

    /**
     * @param maxLen
     * @param buff
     */
    private void fillRows(int[] maxLen, StringBuffer buff) {
        char[] labelBuff;
        String label;
        for (int i = 0; i < filteredRows.size(); i++) {
        	for (int j = 0; j < _columnNames.length; j++) {
                labelBuff = new char[maxLen[j] + COLUMN_SPACE];
        		if (j > 0) {
        			label = filteredRows.get(i).getValueAt(j).toString();
        		} else {
        			label = String.valueOf(i + 1);
        		}
        		Arrays.fill(labelBuff, ' ');
        		String strLabel = new String(labelBuff);
        		StringBuffer buf = new StringBuffer(strLabel);
        		buf.replace(0, label.length(), label);
        		buff.append(buf.toString());
        	}
        	buff.append("\r\n");
        }
    }

	/**
     * @param buff
     */
    private void fillHeader(StringBuffer buff,int[] maxLen) {
        char[] labelBuff;
        for (int j = 0; j < _columnNames.length; j++) {
            labelBuff = new char[maxLen[j] + COLUMN_SPACE];
            Arrays.fill(labelBuff, ' ');
            String strLabel = new String(labelBuff);
            StringBuffer buf = new StringBuffer(strLabel);
            buf.replace(0, _columnNames[j].length(), _columnNames[j]);
            buff.append(buf.toString());
        }
        buff.append("\r\n");
        buff.append("\r\n");
    }

    /**
     * @return
     */
    private int[] getMaxLengthContent() {
        
        int[] maxLen =new int[_columnNames.length];
        Arrays.fill(maxLen,0);
        String label;
        
        for (int j = 0; j < _columnNames.length; j++) {
            if (_columnNames[j].length() > maxLen[j])
                maxLen[j] = _columnNames[j].length();
        }
        for (int i = 0; i < filteredRows.size(); i++) {
            for (int j = 0; j < _columnNames.length; j++) {
                label = filteredRows.get(i).getValueAt(j).toString();
                if (label.length() > maxLen[j])
                    maxLen[j] = label.length();
            }
        }
        return maxLen;
    }

    public void repaintShow() {
		filterRows(1);
		table.setVisible(false);
		table.setVisible(true);
	}

	public void clearRows() {
		filteredRows.clear();
		rows.clear();
	}

	public void addRow(UserBox urow) {
		rows.add(urow);
	}

	public ArrayList getRowst() {
		return filteredRows;
	}

	public JTable getTable() {
		return table;
	}

	public TablePanel(String panelName, MailChecker panel,int mnemonic, String[] columnNames, TotalFactory totalFactory) {
		_columnNames = columnNames;
		_totalFactory = totalFactory;
		_mainPanel = panel;
		setLayout(new BorderLayout());
		AdvancedTableModel amodel = new AdvancedTableModel(filteredRows, _columnNames);
		table = new JTable(amodel);
		amodel.setTable(table);
		table.getColumnModel().getColumn(0).setMaxWidth(50);
		JScrollPane scrollPane = new JScrollPane(table);
		add(scrollPane, BorderLayout.CENTER);
		add(ToolBarWrapper.wrap(new GridToolBar(_mainPanel.getMainMenu(),panelName,mnemonic)), BorderLayout.NORTH);
		//_sbar = new TableStatusBar();
		//add(_sbar, BorderLayout.SOUTH);
	}

	private class GridToolBar extends MenuToolBar {

		/**
		 * Comment for <code>serialVersionUID</code>
		 */
		private static final long serialVersionUID = 3258688823398250289L;

		public GridToolBar(JMenuBar menuBar,String tbName,int mnemonic) {
			super(menuBar, tbName, mnemonic);
			addButton(new RefreshButton());
			addButton(new PrintButton());
			addButton(new FilterButton());
			addSeparator();
			add(filterField);
			addSeparator();
			addButton(new DefaultButton());
			addButton(new SaveButton());
			setFloatable(false);
		}

		private class SaveButton extends ToolButton {
			/**
			 * Comment for <code>serialVersionUID</code>
			 */
			private static final long serialVersionUID = 3257850986847286833L;

			public SaveButton() {
				super(IconMaker.getIcon16(IconMaker.SAVE_ICON), "Save");
				createMenuItem("Save",KeyEvent.VK_1, -1);
			}

			public void runClick() {
				new TextEditDialog(TablePanel.this, "Save the Process Results", saveTableToText(),
						TextEditDialog.SAVE_DIALOG);
			}
		}

		private class RefreshButton extends ToolButton {

			/**
			 * Comment for <code>serialVersionUID</code>
			 */
			private static final long serialVersionUID = 3258133565714872373L;

			public RefreshButton() {
				super("Refresh", IconMaker.getIcon16(IconMaker.REFRESH_ICON));
				createMenuItem("Refresh",KeyEvent.VK_2, -1);
			}

			public void runClick() {
				repaintShow();
			}
		}

		private class PrintButton extends ToolButton {

			/**
			 * Comment for <code>serialVersionUID</code>
			 */
			private static final long serialVersionUID = 3257566221992866104L;

			public PrintButton() {
				super("Print", IconMaker.getIcon16(IconMaker.PRINT_ICON));
				createMenuItem("Print",KeyEvent.VK_3, -1);
			}

			public void runClick() {
				printTable();
			}
		}

		private class FilterButton extends ToolButton {

			/**
			 * Comment for <code>serialVersionUID</code>
			 */
			private static final long serialVersionUID = 3689915084788871990L;

			public FilterButton() {
				super("Filter", IconMaker.getIcon16(IconMaker.FILTER_ICON));
				createMenuItem("Filter",KeyEvent.VK_4, -1);
			}

			public void runClick() {
				int col = table.getSelectedColumn();
				if (col == -1)
					col = 1;
				if (col > 2)
					col = 1;
				if (col < 1)
					col = 1;
				filterRows(col);
				table.revalidate();
			}
		}

		private class DefaultButton extends ToolButton {

			/**
			 * Comment for <code>serialVersionUID</code>
			 */
			private static final long serialVersionUID = 3545519495343912754L;

			public DefaultButton() {
				super("Default", IconMaker.getIcon16(IconMaker.DEFAULT_ICON));
				createMenuItem("Default",KeyEvent.VK_5, -1);
			}

			public void runClick() {
				filterField.setText("");
				filteredRows.clear();
				filteredRows.addAll(rows);
				table.revalidate();
			}
		}

	}

	/*private class TableStatusBar extends JPanel {

		
		private static final long serialVersionUID = 3618984481563161139L;

		public StatusPanel _rows;

		public StatusPanel _totalIn;

		public StatusPanel _totalOut;

		public TableStatusBar() {
			setLayout(new BorderLayout());
			_rows = new StatusPanel("  Rows: ", 150);
			_totalIn = new StatusPanel("  TotalIncoming: ", 200);
			_totalOut = new StatusPanel("  TotalOutgoing: ");
			Box bh = Box.createHorizontalBox();
			bh.add(_rows);
			bh.add(_totalIn);
			bh.add(_totalOut);
			add(bh, BorderLayout.WEST);
			add(_totalOut, BorderLayout.CENTER);
		}

		public class StatusPanel extends JPanel {

			
			private static final long serialVersionUID = 3257288032632780595L;

			private JLabel lab = new JLabel();

			private String pref;

			public StatusPanel(String prefix, int width) {
				pref = prefix;
				lab.setFont(new Font("Tahoma", Font.BOLD, 11));
				lab.setText(pref);
				Box bvv = Box.createVerticalBox();
				bvv.add(Box.createVerticalStrut(16));
				setLayout(new BorderLayout());
				setBorder(BorderFactory.createEtchedBorder());
				Box bh = Box.createHorizontalBox();
				bh.add(Box.createHorizontalStrut(width));
				add(bh, BorderLayout.SOUTH);
				add(bvv, BorderLayout.WEST);
				add(lab, BorderLayout.CENTER);
			}

			public StatusPanel(String prefix) {
				pref = prefix;
				lab.setFont(new Font("Tahoma", Font.BOLD, 11));
				lab.setText(pref);
				Box bvv = Box.createVerticalBox();
				bvv.add(Box.createVerticalStrut(16));
				setLayout(new BorderLayout());
				setBorder(BorderFactory.createEtchedBorder());
				Box bh = Box.createHorizontalBox();
				bh.add(Box.createHorizontalGlue());
				add(bh, BorderLayout.SOUTH);
				add(bvv, BorderLayout.WEST);
				add(lab, BorderLayout.CENTER);
			}

			public void setText(String text) {
				//System.out.println(text+"m------------");
				lab.setText(pref + text);
			}
		}
	}*/
}
