/*
 * Created on 26.12.2004
 */
package com.dialogs;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Frame;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.prefs.Preferences;

import javax.swing.BorderFactory;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;

import com.common.IconMaker;
import com.common.PositionUtilities;
import com.common.ToolButton;

/**
 * @author nemo
 */
public class TextEditDialog extends JDialog{
	
	/**
	 * Comment for <code>serialVersionUID</code>
	 */
	private static final long serialVersionUID = 3258131340838123829L;
	
	public static final int EDIT_DIALOG = 0;
	
	public static final int SAVE_DIALOG = 1;
	
	private JTextArea  _text = new JTextArea();
	
	private boolean isSaved = false;
	
	private int _dlgType = 0;
	
	private class OKButton extends ToolButton{

		/**
		 * Comment for <code>serialVersionUID</code>
		 */
		private static final long serialVersionUID = 3762816008473555506L;

		public OKButton() {
			super("OK",IconMaker.getIcon16(IconMaker.OK_ICON));
		}
		
		public void runClick() {
			isSaved = true;
			TextEditDialog.this.dispose();
		}
		
	}
	
	private class CancelButton extends ToolButton{

		/**
		 * Comment for <code>serialVersionUID</code>
		 */
		private static final long serialVersionUID = 3258131362279076150L;

		public CancelButton() {
			super("Close",IconMaker.getIcon16(IconMaker.DEFAULT_ICON));
		}
		
		public void runClick() {
			TextEditDialog.this.dispose();
		}
		
	}
	
	private class SaveButton extends ToolButton{
		/**
		 * Comment for <code>serialVersionUID</code>
		 */
		private static final long serialVersionUID = 3618983377722816822L;

		public SaveButton() {
			super("Save",IconMaker.getIcon16(IconMaker.SAVE_ICON));
		}
		
		public void runClick() {
			JFileChooser chooser = new JFileChooser();
			chooser.setDialogType(JFileChooser.FILES_ONLY);
			chooser.setSelectedFile(new File(getUserFileName()));
			int returnVal = chooser.showSaveDialog(TextEditDialog.this);
			if (returnVal == JFileChooser.APPROVE_OPTION) {
                //saveUserFilename(chooser.getSelectedFile());
				setFileContent(chooser.getSelectedFile());
				TextEditDialog.this.dispose();
			}
		}

        /**
         * @return
         */
        private String getUserFileName() {
            //Preferences prefs = Preferences.userNodeForPackage(this.getClass());
            //return prefs.get("userFileName", ".txt");
            return ".txt";
        }

        /**
         * @param selectedFile
         */
        private void saveUserFilename(File selectedFile) {
            Preferences prefs = Preferences.userNodeForPackage(this.getClass());
            prefs.put("userFileName", selectedFile.getAbsolutePath());           
        }
		
	}
	
	private void  setFileContent(File fitem) {
		try {
			FileOutputStream fin = new FileOutputStream(fitem);
			BufferedWriter buf = new BufferedWriter(new OutputStreamWriter(fin));
			try {
				buf.write(_text.getText());
				buf.flush();
				buf.close();
			} catch (IOException e1) {
				e1.printStackTrace();
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
	}
	
	public JTextArea getText() {
		return _text;
	}
	private JPanel getButtonPanel() {
		JPanel panel = new JPanel();
		panel.setLayout(new FlowLayout());
		if(_dlgType == EDIT_DIALOG) {
		panel.add(new OKButton());
		}else {
			_text.setFont(new Font("Courier New",Font.PLAIN,14));
		}
		panel.add(new SaveButton());
		panel.add(new CancelButton());
		panel.setBorder(BorderFactory.createEtchedBorder());
		return panel;
	}
	
	private JPanel getMainPanel() {
		JPanel panel = new JPanel();
		panel.setLayout(new BorderLayout());
		panel.add(new JScrollPane(_text),BorderLayout.CENTER);
		panel.setBorder(BorderFactory.createEtchedBorder());
		return panel;
	}
	
	public TextEditDialog(Component owner,String title,String editText) {
		super((Frame)SwingUtilities.getWindowAncestor(owner),true);
		setTitle(title);
		_text.setText(editText);
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(getButtonPanel(),BorderLayout.SOUTH);
		getContentPane().add(getMainPanel(),BorderLayout.CENTER);
		setSize(400,500);
		PositionUtilities.setCentered(this);
		setVisible(true);
	}
	
	public TextEditDialog(Component owner,String title,String editText,int dlgType) {
		super((Frame)SwingUtilities.getWindowAncestor(owner),true);
		_dlgType = dlgType;
		setTitle(title);
		_text.setText(editText);
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(getButtonPanel(),BorderLayout.SOUTH);
		getContentPane().add(getMainPanel(),BorderLayout.CENTER);
		if(_dlgType == EDIT_DIALOG)
		setSize(400,500);
		else
			setSize(780,500);
		PositionUtilities.setCentered(this);
		setVisible(true);
	}
	

	public boolean isSaved() {
		return isSaved;
	}
}
