package com.panels;
import java.awt.BorderLayout;
import java.awt.event.KeyEvent;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.StringReader;
import java.util.HashMap;

import javax.swing.JFileChooser;
import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;

import com.MailChecker;
import com.ToolBarWrapper;
import com.common.FileDomBox;
import com.common.IconMaker;
import com.common.MenuToolBar;
import com.common.ToolButton;
import com.dialogs.TextEditDialog;

/*
 * Created on 16.12.2004
 */

/**
 * @author nemo
 */
public class DomainsListPan extends JPanel {
	
	/**
	 * Comment for <code>serialVersionUID</code>
	 */
	private static final long serialVersionUID = 3977017327533439028L;

	private String _fileContent="";
	
	private File currDir;
	
	private String[] columnNames = {"Check","Domain Name"};

	private FileTableModel tableModel;

	private JTable table;
	
	private MailChecker _mainPanel;
	
	private HashMap <String ,FileDomBox> rows;
	
	public void repaintShow() {
		table.setVisible(false);
		table.setVisible(true);
	}

	private class FileTableModel extends AbstractTableModel {

		/**
		 * Comment for <code>serialVersionUID</code>
		 */
		private static final long serialVersionUID = 3618695301414859314L;

		public String getColumnName(int column) {
			return columnNames[column];
		}

		public int getRowCount() {
			return rows.size();
		}

		public int getColumnCount() {
			return columnNames.length;
		}

		public Object getValueAt(int rowIndex, int columnIndex) {
			try {
				FileDomBox row =  (FileDomBox) rows.values().toArray()[rowIndex];
				return row.getValueAt(columnIndex);
			} catch (RuntimeException e) {
				e.printStackTrace();
			}
			return "?";
		}

		public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
			FileDomBox row = (FileDomBox) rows.values().toArray()[rowIndex];
			row.setValueAt(columnIndex,aValue);
			fireTableCellUpdated(rowIndex, columnIndex);
		}
		
		public Class <?> getColumnClass(int c) {
            return getValueAt(0, c).getClass();
        }

        public boolean isCellEditable(int row, int col) {
            if (col < 1) 
                return true;
                return false;
        }


	}
	
	public DomainsListPan(MailChecker owner) {
		_mainPanel = owner;
		rows = _mainPanel.getDataSave().getDomainsFilter();
		rows.clear();
		setLayout(new BorderLayout());
		tableModel = new FileTableModel();
		table = new JTable(tableModel);
		table.getColumnModel().getColumn(0).setMaxWidth(50);
		JScrollPane scrollPane = new JScrollPane(table);
		add(scrollPane, BorderLayout.CENTER);
		add(ToolBarWrapper.wrap(new GridToolBar(_mainPanel.getMainMenu(),"Local Domains",KeyEvent.VK_D)), BorderLayout.NORTH);
	}
	
	public void cleanFiles() {
		rows.clear();
		table.revalidate();
	}
	
	
	private class GridToolBar extends MenuToolBar {

		/**
		 * Comment for <code>serialVersionUID</code>
		 */
		private static final long serialVersionUID = 3257005475274043700L;

		public GridToolBar(JMenuBar menuBar,String tbName,int mnemonic) {
			super(menuBar, tbName, mnemonic);
			addButton(new LoadButton());
			addButton(new DeleteButton());
			addButton(new ClearButton());
			addButton(new RefreshButton());
			addButton(new EditButton());
			addButton(new SaveButton());
			setFloatable(false);
		}
		
		private class SaveButton extends ToolButton {

			/**
			 * Comment for <code>serialVersionUID</code>
			 */
			private static final long serialVersionUID = 3977297707343687986L;

			public SaveButton() {
				super(IconMaker.getIcon16(IconMaker.SAVE_ICON),"Save");
				createMenuItem("Save",KeyEvent.VK_1, -1);
			}

			public void runClick() {
				JFileChooser chooser = new JFileChooser();
				if(currDir != null)
					chooser.setSelectedFile(currDir);
				chooser.setDialogType(JFileChooser.FILES_ONLY);
				chooser.setMultiSelectionEnabled(true);
				int returnVal = chooser.showSaveDialog(_mainPanel);
				if (returnVal == JFileChooser.APPROVE_OPTION) {
					currDir = chooser.getSelectedFile();
					setFileContent(currDir);
				}
			}
		}
		
		private class EditButton extends ToolButton {

			/**
			 * Comment for <code>serialVersionUID</code>
			 */
			private static final long serialVersionUID = 3256727294537118260L;

			public EditButton() {
				super(IconMaker.getIcon16(IconMaker.EDIT_ICON),"Add or Edit");
				createMenuItem("Add or Edit",KeyEvent.VK_2, -1);
			}

			public void runClick() {
				TextEditDialog dlg = new TextEditDialog(_mainPanel,"Editing the Local Domains List",_fileContent);
				if(dlg.isSaved()) {
					String res = "";
					try {
						rows.clear();
						_fileContent = dlg.getText().getText();
						StringReader strreader = new StringReader(dlg.getText().getText());
						BufferedReader buf = new BufferedReader(strreader);
						res = buf.readLine();
						while (res != null) {
							FileDomBox dbox = new FileDomBox(res,table);
							rows.put(res,dbox);
							res = buf.readLine();
						}
						repaintShow();
					} catch (IOException e1) {
						e1.printStackTrace();
					}
				}
			}
		}
		
		private class RefreshButton extends ToolButton {

			/**
			 * Comment for <code>serialVersionUID</code>
			 */
			private static final long serialVersionUID = 3258416122989588528L;

			public RefreshButton() {
				super(IconMaker.getIcon16(IconMaker.REFRESH_ICON),"Refresh");
				createMenuItem("Refresh",KeyEvent.VK_3, -1);
			}

			public void runClick() {
				repaintShow();
			}
		}

		private class LoadButton extends ToolButton {

			/**
			 * Comment for <code>serialVersionUID</code>
			 */
			private static final long serialVersionUID = 3256445819677849910L;

			public LoadButton() {
				super(IconMaker.getIcon16(IconMaker.LOAD_ICON),"Load");
				createMenuItem("Load",KeyEvent.VK_4, -1);
			}
			
			public void runClick() {
				JFileChooser chooser = new JFileChooser();
				if(currDir != null)
					chooser.setSelectedFile(currDir);
				chooser.setDialogType(JFileChooser.FILES_ONLY);
				chooser.setMultiSelectionEnabled(true);
				int returnVal = chooser.showOpenDialog(_mainPanel);
				if (returnVal == JFileChooser.APPROVE_OPTION) {
					currDir = chooser.getSelectedFile();
					getFileContent(currDir);
					repaintShow();
				}
			}
		}
		
		private void  getFileContent(File fitem) {
			try {
				FileInputStream fin = new FileInputStream(fitem);
				BufferedReader buf = new BufferedReader(new InputStreamReader(fin));
				String res = "";
				try {
					res = buf.readLine();
					StringBuffer stringB = new StringBuffer("");
					while (res != null) {
						if(res.length()<2) {
							res = buf.readLine();
							continue;
						}
						FileDomBox dbox = new FileDomBox(res,table);
						stringB.append(res+"\n");
						rows.put(res,dbox);
						res = buf.readLine();
					}
					_fileContent = stringB.toString();
					buf.close();
				} catch (IOException e1) {
					e1.printStackTrace();
				}
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			}
		}
		
		
		private void  setFileContent(File fitem) {
			try {
				FileOutputStream fin = new FileOutputStream(fitem);
				BufferedWriter buf = new BufferedWriter(new OutputStreamWriter(fin));
				FileDomBox row;
				try {
					for(int i = 0;i<rows.size();i++) {
						row = (FileDomBox) rows.values().toArray()[i];
						buf.write(row.getValueAt(1).toString()+"\n");
					}
					buf.flush();
					buf.close();
				} catch (IOException e1) {
					e1.printStackTrace();
				}
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			}
		}
		
		private class DeleteButton extends ToolButton {

			/**
			 * Comment for <code>serialVersionUID</code>
			 */
			private static final long serialVersionUID = 3689070638266856761L;

			public DeleteButton() {
				super(IconMaker.getIcon16(IconMaker.DELETE_ICON),"Delete");
				createMenuItem("Delete",KeyEvent.VK_5, -1);
			}
			
			public void runClick() {
				if(table.getSelectedRow() == -1)return;
				Object delObj = tableModel.getValueAt(table.getSelectedRow(),1);
				rows.remove(delObj);
				repaintShow();
			}
		}
		
		private class ClearButton extends ToolButton {

			/**
			 * Comment for <code>serialVersionUID</code>
			 */
			private static final long serialVersionUID = 3256444715736969525L;

			public ClearButton() {
				super(IconMaker.getIcon16(IconMaker.CLEAR_ICON),"Clear");
				createMenuItem("Clear",KeyEvent.VK_6, -1);
			}
			
			public void runClick() {
				cleanFiles();
			}
		}

	}

}
