/*
 * Created on 12.12.2004
 */
package com.panels;
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.io.File;
import java.util.ArrayList;

import javax.swing.JFileChooser;
import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;

import com.MailChecker;
import com.ToolBarWrapper;
import com.common.FileBox;
import com.common.IconMaker;
import com.common.MenuToolBar;
import com.common.ToolButton;


/**
 * @author nemo
 */
public class FileListPan extends JPanel {
	
	/**
	 * Comment for <code>serialVersionUID</code>
	 */
	private static final long serialVersionUID = 3761124950983127863L;

	public String _path;
	
	private File[] _files;
	
	private String[] columnNames = {"NN", "Check","Name"};

	private FileTableModel tableModel;

	private JTable table;
	
	private File currDir;
	
	private MailChecker _mainPanel;
	
	private ArrayList <FileBox> rows;
	
	public void repaintShow() {
		table.setVisible(false);
		table.setVisible(true);
	}

	private class FileTableModel extends AbstractTableModel {

		/**
		 * Comment for <code>serialVersionUID</code>
		 */
		private static final long serialVersionUID = 3258416114466894393L;

		public String getColumnName(int column) {
			return columnNames[column];
		}

		public int getRowCount() {
			return rows.size();
		}

		public int getColumnCount() {
			return columnNames.length;
		}

		public Object getValueAt(int rowIndex, int columnIndex) {
			try {
				FileBox row =  rows.get(rowIndex);
				return row.getValueAt(columnIndex);
			} catch (RuntimeException e) {
				e.printStackTrace();
			}
			return "?";
		}

		public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
			FileBox row = rows.get(rowIndex);
			row.setValueAt(columnIndex,aValue);
			fireTableCellUpdated(rowIndex, columnIndex);
		}
		
		public Class <?> getColumnClass(int c) {
            return getValueAt(0, c).getClass();
        }

        public boolean isCellEditable(int row, int col) {
            if (col < 1) 
                return false;
                return true;
        }


	}
	
	public FileListPan(MailChecker owner) {
		_mainPanel = owner;
		rows = _mainPanel.getDataSave().getFileRows();
		setLayout(new BorderLayout());
		tableModel = new FileTableModel();
		//TableDataManager sorter = new TableDataManager(tableModel);
		table = new JTable(tableModel);
		//sorter.setTableHeader(table.getTableHeader());
		table.getColumnModel().getColumn(0).setMaxWidth(50);
		table.getColumnModel().getColumn(1).setMaxWidth(60);
		//add(table.getTableHeader(), BorderLayout.PAGE_START);
		JScrollPane scrollPane = new JScrollPane(table);
		add(scrollPane, BorderLayout.CENTER);
		add(ToolBarWrapper.wrap(new GridToolBar(_mainPanel.getMainMenu(),"Files",KeyEvent.VK_F)), BorderLayout.NORTH);
	}
	
	public void cleanFiles() {
		rows.clear();
		repaintShow();
	}
	
	public void setFiles(File[] files) {
		_files = files;
		for(int i = 0;i<_files.length;i++) {
			rows.add(new FileBox(rows.size()+1,_files[i].getName(),_files[i],table));
		}
		 repaintShow();
	}
	
	private class GridToolBar extends MenuToolBar {

		/**
		 * Comment for <code>serialVersionUID</code>
		 */
		private static final long serialVersionUID = 3833184748339213107L;

		public GridToolBar(JMenuBar menuBar,String tbName,int mnemonic) {
			super(menuBar, tbName, mnemonic);
			addButton(new LoadButton());
			addButton(new DeleteButton());
			addButton(new ClearButton());
			addButton(new RefreshButton());
			setFloatable(false);
		}
		
		private class RefreshButton extends ToolButton {

			/**
			 * Comment for <code>serialVersionUID</code>
			 */
			private static final long serialVersionUID = 3761403097359922488L;

			public RefreshButton() {
				super(IconMaker.getIcon16(IconMaker.REFRESH_ICON),"Refresh");
				createMenuItem("Refresh",KeyEvent.VK_R, -1);
			}

			public void runClick() {
				repaintShow();
			}
		}

		private class LoadButton extends ToolButton {

			/**
			 * Comment for <code>serialVersionUID</code>
			 */
			private static final long serialVersionUID = 3834024753781617971L;

			public LoadButton() {
				super(IconMaker.getIcon16(IconMaker.LOAD_ICON),"Load");
				createMenuItem("Load",KeyEvent.VK_9,ActionEvent.ALT_MASK);
			}
			
			public void runClick() {
				JFileChooser chooser = new JFileChooser();
				if(currDir != null)
					chooser.setSelectedFile(currDir);
				chooser.setDialogType(JFileChooser.FILES_ONLY);
				chooser.setMultiSelectionEnabled(true);
				int returnVal = chooser.showOpenDialog(_mainPanel);
				if (returnVal == JFileChooser.APPROVE_OPTION) {
					currDir = chooser.getSelectedFile();
					setFiles(chooser.getSelectedFiles());
				}
			}
		}
		
		private class DeleteButton extends ToolButton {

			/**
			 * Comment for <code>serialVersionUID</code>
			 */
			private static final long serialVersionUID = 3257008765268931891L;

			public DeleteButton() {
				super(IconMaker.getIcon16(IconMaker.DELETE_ICON),"Delete");
				createMenuItem("Delete",KeyEvent.VK_D, -1);
			}
			
			public void runClick() {
				if(table.getSelectedRow() == -1)return;
				rows.remove(table.getSelectedRow());
				repaintShow();
			}
		}
		
		private class ClearButton extends ToolButton {

			/**
			 * Comment for <code>serialVersionUID</code>
			 */
			private static final long serialVersionUID = 3761404209790464565L;

			public ClearButton() {
				super(IconMaker.getIcon16(IconMaker.CLEAR_ICON),"Clear");
				createMenuItem("Clear",KeyEvent.VK_C, -1);
			}
			
			public void runClick() {
				cleanFiles();
			}
		}

	}


}
