/*
 * Created on 20.12.2004
 */
package com.panels;

import java.awt.BorderLayout;
import java.awt.event.KeyEvent;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.StringReader;
import java.util.HashMap;

import javax.swing.JFileChooser;
import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;

import com.MailChecker;
import com.ToolBarWrapper;
import com.common.FileDomBox;
import com.common.IconMaker;
import com.common.MenuToolBar;
import com.common.ToolButton;
import com.common.UserFileDomBox;
import com.dialogs.TextEditDialog;

/**
 * @author nemo
 */
public class UserGroupPan extends JPanel {
	
	/**
	 * Comment for <code>serialVersionUID</code>
	 */
	private static final long serialVersionUID = 3690753982062670647L;

	public String _path;
	
	private File currDir;
	
	private String[] columnNames = {"Check","Domain","User"};

	private FileTableModel tableModel;

	private JTable table;
	
	private MailChecker _mainPanel;
	
	private HashMap <String ,UserFileDomBox> rows;
	
	private HashMap <String ,FileDomBox> domains;
	
	private String _fileContent="";
	
	public void repaintShow() {
		table.setVisible(false);
		table.setVisible(true);
	}

	private class FileTableModel extends AbstractTableModel {

		/**
		 * Comment for <code>serialVersionUID</code>
		 */
		private static final long serialVersionUID = 3617569401457684790L;

		public String getColumnName(int column) {
			return columnNames[column];
		}

		public int getRowCount() {
			return rows.size();
		}

		public int getColumnCount() {
			return columnNames.length;
		}

		public Object getValueAt(int rowIndex, int columnIndex) {
			try {
				FileDomBox row =  (FileDomBox)rows.values().toArray()[rowIndex];
				return row.getValueAt(columnIndex);
			} catch (RuntimeException e) {
				e.printStackTrace();
			}
			return "?";
		}

		public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
			FileDomBox row = (FileDomBox) rows.values().toArray()[rowIndex];
			row.setValueAt(columnIndex,aValue);
			fireTableCellUpdated(rowIndex, columnIndex);
		}
		
		public Class <?> getColumnClass(int c) {
            return getValueAt(0, c).getClass();
        }

        public boolean isCellEditable(int row, int col) {
            if (col < 1) 
            	return true;
                return false;
                
        }


	}
	
	public UserGroupPan(MailChecker owner) {
		_mainPanel = owner;
		rows = _mainPanel.getDataSave().getUsersFilter();
		domains = _mainPanel.getDataSave().getDomainsFilter();
		rows.clear();
		domains.clear();
		setLayout(new BorderLayout());
		tableModel = new FileTableModel();
		table = new JTable(tableModel);
		table.getColumnModel().getColumn(0).setMaxWidth(50);
		JScrollPane scrollPane = new JScrollPane(table);
		add(scrollPane, BorderLayout.CENTER);
		add(ToolBarWrapper.wrap(new GridToolBar(_mainPanel.getMainMenu(),"Local Users",KeyEvent.VK_L)), BorderLayout.NORTH);
	}
	
	public void cleanFiles() {
		rows.clear();
		table.revalidate();
	}
	
	public boolean isUserFiltered(String userName) {
		if(rows.size() == 0)return true;
		UserFileDomBox ufd =  rows.get(userName);
		if(ufd != null) 
			return ufd.isChecked();
		return false;
	}
	
	private class GridToolBar extends MenuToolBar {

		/**
		 * Comment for <code>serialVersionUID</code>
		 */
		private static final long serialVersionUID = 3258417231124510772L;

		public GridToolBar(JMenuBar menuBar,String tbName,int mnemonic) {
			super(menuBar, tbName, mnemonic);
			addButton(new LoadButton());
			addButton(new DeleteButton());
			addButton(new ClearButton());
			addButton(new RefreshButton());
			addButton(new EditButton());
			addButton(new SaveButton());
			setFloatable(false);
		}
		
		private class SaveButton extends ToolButton {
			/**
			 * Comment for <code>serialVersionUID</code>
			 */
			private static final long serialVersionUID = 3257845463586779192L;

			public SaveButton() {
				super(IconMaker.getIcon16(IconMaker.SAVE_ICON),"Save");
				createMenuItem("Save",KeyEvent.VK_1, -1);
			}

			public void runClick() {
				JFileChooser chooser = new JFileChooser();
				if(currDir != null)
					chooser.setSelectedFile(currDir);
				chooser.setDialogType(JFileChooser.FILES_ONLY);
				chooser.setMultiSelectionEnabled(true);
				int returnVal = chooser.showSaveDialog(_mainPanel);
				if (returnVal == JFileChooser.APPROVE_OPTION) {
					currDir = chooser.getSelectedFile();
					setFileContent(currDir);
				}
			}
		}
		
		private class EditButton extends ToolButton {
			/**
			 * Comment for <code>serialVersionUID</code>
			 */
			private static final long serialVersionUID = 3257845463569937975L;

			public EditButton() {
				super(IconMaker.getIcon16(IconMaker.EDIT_ICON),"Add or Edit");
				createMenuItem("Add or Edit",KeyEvent.VK_2, -1);
			}

			public void runClick() {
				TextEditDialog dlg = new TextEditDialog(_mainPanel,"Editing the Local Users List",_fileContent);
				if(dlg.isSaved()) {
					String res = "";
					try {
						rows.clear();
						domains.clear();
						_fileContent = dlg.getText().getText();
						StringReader strreader = new StringReader(dlg.getText().getText());
						BufferedReader buf = new BufferedReader(strreader);
						res = buf.readLine();
						while (res != null) {
							String[] arres = res.split("@");
							UserFileDomBox ufd = new UserFileDomBox(arres[0],arres[1],table);
							rows.put(arres[0],ufd);
							FileDomBox dbox = new FileDomBox(arres[1],table);
							domains.put(arres[1],dbox);
							res = buf.readLine();
						}
						repaintShow();
					} catch (IOException e1) {
						e1.printStackTrace();
					}
				}
			}
		}
		
		private class RefreshButton extends ToolButton {

			/**
			 * Comment for <code>serialVersionUID</code>
			 */
			private static final long serialVersionUID = 4049360816153113398L;

			public RefreshButton() {
				super(IconMaker.getIcon16(IconMaker.REFRESH_ICON),"Refresh");
				createMenuItem("Refresh",KeyEvent.VK_3, -1);
			}

			public void runClick() {
				repaintShow();
			}
		}

		private class LoadButton extends ToolButton {

			/**
			 * Comment for <code>serialVersionUID</code>
			 */
			private static final long serialVersionUID = 4049352028667262519L;

			public LoadButton() {
				super(IconMaker.getIcon16(IconMaker.LOAD_ICON),"Load");
				createMenuItem("Load",KeyEvent.VK_4, -1);
			}
			
			public void runClick() {
				JFileChooser chooser = new JFileChooser();
				if(currDir != null)
					chooser.setSelectedFile(currDir);
				chooser.setDialogType(JFileChooser.FILES_ONLY);
				chooser.setMultiSelectionEnabled(true);
				int returnVal = chooser.showOpenDialog(_mainPanel);
				if (returnVal == JFileChooser.APPROVE_OPTION) {
					currDir = chooser.getSelectedFile();
					getFileContent(currDir);
					repaintShow();
				}
			}
		}
		
		private void  getFileContent(File fitem) {
			try {
				FileInputStream fin = new FileInputStream(fitem);
				BufferedReader buf = new BufferedReader(new InputStreamReader(fin));
				String res = "";
				try {
					res = buf.readLine();
					StringBuffer stringB = new StringBuffer("");
					while (res != null) {
						if(res.length()<6) {
							res = buf.readLine();
							continue;
						}
						String[] arres = res.split("@");
						UserFileDomBox ufd = new UserFileDomBox(arres[0],arres[1],table);
						rows.put(arres[0],ufd);
						stringB.append(res+"\n");
						FileDomBox dbox = new FileDomBox(arres[1],table);
						domains.put(arres[1],dbox);
						res = buf.readLine();
					}
					_fileContent = stringB.toString();
					buf.close();
				} catch (IOException e1) {
					e1.printStackTrace();
				}
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			}
		}
		
		private void  setFileContent(File fitem) {
			try {
				FileOutputStream fin = new FileOutputStream(fitem);
				BufferedWriter buf = new BufferedWriter(new OutputStreamWriter(fin));
				UserFileDomBox row;
				try {
					for(int i = 0;i<rows.size();i++) {
						row = (UserFileDomBox) rows.values().toArray()[i];
						buf.write(row.getValueAt(2).toString()+"@"+row.getValueAt(1).toString()+"\n");
					}
					buf.flush();
					buf.close();
				} catch (IOException e1) {
					e1.printStackTrace();
				}
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			}
		}
		
		private class DeleteButton extends ToolButton {

			/**
			 * Comment for <code>serialVersionUID</code>
			 */
			private static final long serialVersionUID = 3257290210214491702L;

			public DeleteButton() {
				super(IconMaker.getIcon16(IconMaker.DELETE_ICON),"Delete");
				createMenuItem("Delete",KeyEvent.VK_5, -1);
			}
			
			public void runClick() {
				if(table.getSelectedRow() == -1)return;
				Object delObj = tableModel.getValueAt(table.getSelectedRow(),2);
				rows.remove(delObj);
				repaintShow();
			}
		}
		
		private class ClearButton extends ToolButton {

			/**
			 * Comment for <code>serialVersionUID</code>
			 */
			private static final long serialVersionUID = 3834587690900141369L;

			public ClearButton() {
				super(IconMaker.getIcon16(IconMaker.CLEAR_ICON),"Clear");
				createMenuItem("Clear",KeyEvent.VK_6, -1);
			}
			
			public void runClick() {
				cleanFiles();
			}
		}

	}

}

